import {generateClientApi} from './gen'
import {writeFileSync} from "fs";

const decorators = {
    Get: `return new Promise((resolve) => {this.api.get('{url}').subscribe((data: any) => {resolve});
});
`,
    Post: `
    return new Promise((resolve) => {
      this.api.post('{url}'{body})
      .subscribe((data: any) => {resolve});
})`,

    Put: `
    return new Promise((resolve) => {
      this.api.put('{url}'{body})
      .subscribe((data: any) => {resolve});
})`,
};

const httpServiceTemplate = `
import {Injectable} from '@angular/core';
import { HttpClient } from '@angular/common/http';
import { catchError } from 'rxjs/operators';
import { throwError } from 'rxjs';
import {ActivatedRoute, Router} from '@angular/router';
@Injectable()
export class APIService {
    constructor(
        private httpClient: HttpClient,
        private router: Router,
        private route: ActivatedRoute,
        @Injectable() private BASEURL = '') {
        this.BASEURL += '/' + this.route.snapshot.params.projectName + '/';
    }
    handleError(error) {
        let errorMessage = '';
        if (error.status === 403) {
            return this.router.navigate(['/login' + window.location.pathname, {}]);
        }
        if (error.error instanceof ErrorEvent) {
            // client-side error
            errorMessage = 'Error:' + error.error.message;
        } else {
            // server-side error
            errorMessage = 'Error Code:' + error.status + 'Message:' + error.message;
        }
        return throwError(errorMessage);
    }
    get(url) {
        return this.httpClient.get(this.BASEURL + url, { withCredentials: true }).pipe(
            catchError(err => this.handleError(err)),
        );
    }
    post(url, body?) {
        return this.httpClient.post(this.BASEURL + url, body, { withCredentials: true }).pipe(
            catchError(err => this.handleError(err)),
        );
    }

    put(url, body?) {
      return this.httpClient.put(this.BASEURL + url, body, { withCredentials: true }).pipe(
        catchError(err => this.handleError(err)),
      );
    }
}
`;

export const generateAngularClientApi = (clientPath :string, serverPath: string) =>{
    const files =  generateClientApi({ clientPath, decorators, httpServiceTemplate, serverPath });
    files.forEach(file => {
        file.addImportDeclaration({
            namedImports: ['Injectable'],
            moduleSpecifier: '@angular/core',
        });
        file.getClasses()[0].addDecorator({ name: 'Injectable', arguments: [] });
        writeFileSync(clientPath + '/' + file.getBaseName(), file.getText());
    })
}
