import {
    copyFileSync,
    lstatSync,
    mkdirSync,
    readdirSync,
    readFileSync,
    readlinkSync,
    symlinkSync,
    writeFileSync
} from "fs";
import {join} from "path";
import {generateClientApi} from './gen'
const copyDir = function(src, dest) {
    mkdirSync(dest, { recursive: true });
    const files = readdirSync(src);
    for(let i = 0; i < files.length; i++) {
        const current = lstatSync(join(src, files[i]));
        if(current.isDirectory()) {
            copyDir(join(src, files[i]), join(dest, files[i]));
        } else if(current.isSymbolicLink()) {
            const symlink = readlinkSync(join(src, files[i]));
            symlinkSync(symlink, join(dest, files[i]));
        } else {
            copyFileSync(join(src, files[i]), join(dest, files[i]));
        }
    }
};
const decorators = {
    Get: `return new Promise((resolve) => {this.api.get(projectName, '{url}').then((data: any) => {resolve});
});
`,
    Post: `
    return new Promise((resolve) => { this.api.post(projectName, '{url}'{body}).then((data: any) => {resolve});
})`,

    Put: `
    return new Promise((resolve) => {this.api.put(projectName, '{url}'{body}).then((data: any) => {resolve});
})`,
};

let httpServiceTemplate = `
import {promisify} from 'util';
import {Service} from "pn-ts/decorators";
const pn = require('pn-ts')();

@Service()
export class APIService {

    get(url,projectName) {
        return this.request('GET', projectName,  url);
    }

    post(url, projectName) {
        return this.request('POST', projectName, url);
    }

    put(url, projectName) {
        return this.request('PUT', projectName, url);
    }

    async request(method: string, projectName, url) {
        const reqSettings = {method: 'GET', pn.config.Url + url, headers: {'Content-Type': 'application/json',}, json: true};
        try {
            const res = await promisify(pn.utils.serverToServerHttpRequest)(pn.config.serviceName, projectName, reqSettings);
            return JSON.parse(res.httpRequest.body);
        } catch (err) {
            pn.logger.error(
                {
                    err,
                },
                'Error getting data from ' + pn.config.Url,
            );

            throw err;
        }
    }
}
`;

export const generateServerToServerApi = (clientPath :string, serverPath: string, domainPath:string, configPath:string) =>{
    httpServiceTemplate = httpServiceTemplate.replace('pn.config.Url', 'pn.config.' + configPath)
    const files =  generateClientApi({ clientPath, decorators, httpServiceTemplate, serverPath });
    files.forEach(file => {
        file.addImportDeclaration({
            namedImports: ['Service'],
            moduleSpecifier: 'pn-ts/decorators',
        });
        file.getClasses()[0].addDecorator({ name: 'Service', arguments: [] });
        file.getClasses()[0].getMethods().forEach(method =>{
            method.addParameter({name:'projectName', type: 'string'})
        })
        const fileText = file.getText().replace(/domain\//i,'./domain/');
        writeFileSync(clientPath + '/' + file.getBaseName(), fileText);
    })

    copyDir(domainPath, clientPath + '/domain');
}
