export {};
const pn = require('../../src')('authentication-example'); // require('progressive-node')('<app name>');
const jwt = require('jsonwebtoken');
const moment = require('moment');

pn.init(__dirname, 'TEST');

const PRIVATE_KEY = `-----BEGIN RSA PRIVATE KEY-----
MIICWwIBAAKBgQCsmhMZIG2tDkXCwBCOd7bjYs9JtPNCdQQxKehnF3qJrXFjeFaS
3Np75a6vfCLp/lbHRBbaF8Bb5KUGDvRU1XPMIPas2seblAzcaGlHH1RxAwyzu6A4
jzvtlzKhxct9nR8wyVquwoa+pgE4Ryu+CphL68baDI7YGfN9toT7/IrsZwIDAQAB
AoGAXzujzflWimWVmB7hA7qJ7mxWcxSkI8bsyTOWR0ttEQnhxkBfhCjXZAUWnTyh
BvqOZuPqt+ri99DwPKeDqEsd2Fl1lM3k6yYEQtkYrBqJS0d32I1CF4Gh+Y4zncAD
xaxnrzCY5Nz79FqBNSp+n95quJSjzEOekAwOMgXMFC9kfAECQQDvCicw4L9qrTE6
i32JNL7vJ4hCM6y0IWWXgegyJ1tzy0CWxtaxOopXXDQe2ZXcuTXcOyNT4a2SQYxS
hRNEVI8pAkEAuNkqHQhwPz1lpb4IqFKgvcjZ4XKhIAVFDWqjAmmkw/dPoRN3LShJ
cZtj551MOKu5LifMpOPaB7Ug0wWdiZhhDwJAax3pogWUwYEK5mlLwAha52CXttLh
OXcdx173ZkIswR77upUv8AuDotI2qPxoI5j7gc0eb6BZ34hwESqBD8W0wQJABWJm
BKR7SIxMSrCnZefKQNz548RIBzvAECz8BJT5NLAehK2hehJDWWjSVyvQzZHK+ho0
rtToQ/M1uo/eT+S9ZwJALnmldz3pXbHz/VdLxyjPGonBG44czJKRSGwjU1Y09cEs
NXyCImbuA1VSvkljcNkTOPw9wHhN0BIkmtuYaU/fVg==
-----END RSA PRIVATE KEY-----`;
const nock = require('nock');
const chai = require('chai');
const chaiHttp = require('chai-http');
const AuthCache = require('progressive-node//components/authentication/AuthCache');

chai.use(chaiHttp);
chai.should();

describe('components/authentication', () => {
    beforeEach(async () => {
        nock.cleanAll();
        const redis = await pn.db.redis();
        await redis.selectAsync(AuthCache.DB);
        await redis.flushdbAsync();
    });

    describe('v3', () => {
        it('cookie', async () => {
            const data = {
                project_name: 'test',
                applications: [
                    {
                        application_code: 'TEST',
                    },
                ],
            };
            const token = jwt.sign(data, pn.config.authentication.authServer.secret);

            const result = await chai
                .request(pn.express)
                .get('/test/isAuthorized')
                .set('Cookie', `access_token=${token};refresh_token=abc`);

            result.res.text.should.be.equal('OK');
        });

        it('isInternal check', async () => {
            const data = {
                is_internal: true,
            };
            const token = jwt.sign(data, pn.config.authentication.authServer.secret);

            const result = await chai
                .request(pn.express)
                .get('/isInternal')
                .set('Cookie', `access_token=${token};refresh_token=abc`);

            result.res.text.should.be.equal('OK');
        });

        it('cookie with no project name', async () => {
            const data = {
                project_name: 'test',
                applications: [
                    {
                        application_code: 'TEST',
                    },
                ],
            };
            const token = jwt.sign(data, pn.config.authentication.authServer.secret);

            const result = await chai
                .request(pn.express)
                .get('/isAuthorized')
                .set('Cookie', `access_token=${token};refresh_token=abc`);

            result.res.text.should.be.equal('OK');
        });

        it('cookie with bad project name', async () => {
            const data = {
                project_name: 'test',
                applications: [
                    {
                        application_code: 'TEST',
                    },
                ],
            };
            const token = jwt.sign(data, pn.config.authentication.authServer.secret);

            const result = await chai
                .request(pn.express)
                .get('/foo/isAuthorized')
                .set('Cookie', `access_token=${token};refresh_token=abc`);

            result.statusCode.should.equal(401);
            result.body.should.be.deep.equal({ err: 'User does not have permissions for project-foo' });
        });

        it('header', async () => {
            const data = {
                project_name: 'test',
                applications: [
                    {
                        application_code: 'TEST',
                    },
                ],
            };
            const token = jwt.sign(data, pn.config.authentication.authServer.secret);

            const result = await chai
                .request(pn.express)
                .get('/test/isAuthorized')
                .set('auth_token', `${JSON.stringify({ access_token: token, refresh_token: 'abc' })}`);

            result.res.text.should.be.equal('OK');
        });

        it('cookie refresh token', async () => {
            const data = {
                project_name: 'test',
                applications: [
                    {
                        application_code: 'TEST',
                    },
                ],
                exp: Math.floor(Date.now() / 1000) + 60,
            };
            const newToken = jwt.sign(data, pn.config.authentication.authServer.secret);

            nock('http://auth.example.com')
                .post('/auth/v3/token')
                .reply(200, { access_token: newToken, refresh_token: 'cde' });

            const token = jwt.sign({ exp: Math.floor(Date.now() / 1000) - 1 }, pn.config.authentication.authServer.secret);

            const result = await chai
                .request(pn.express)
                .get('/test/isAuthorized')
                .set('Cookie', `access_token=${token};refresh_token=abc`);

            result.res.text.should.be.equal('OK');
        });

        it('header refresh token', async () => {
            const data = {
                project_name: 'test',
                applications: [
                    {
                        application_code: 'TEST',
                    },
                ],
                exp: Math.floor(Date.now() / 1000) + 60,
            };
            const newToken = jwt.sign(data, pn.config.authentication.authServer.secret);

            nock('http://auth.example.com')
                .post('/auth/v3/token')
                .reply(200, { access_token: newToken, refresh_token: 'cde' });

            const token = jwt.sign({ exp: Math.floor(Date.now() / 1000) - 1 }, pn.config.authentication.authServer.secret);

            const result = await chai
                .request(pn.express)
                .get('/test/isAuthorized')
                .set('auth_token', `${JSON.stringify({ access_token: token, refresh_token: 'abc' })}`);

            result.res.text.should.be.equal('OK');
        });
    });

    describe('v4', () => {
        it('cookie', async () => {
            const data = {
                project_name: 'test',
                applications: [
                    {
                        application_code: 'TEST',
                    },
                ],
            };
            const token = jwt.sign(data, PRIVATE_KEY, { algorithm: 'RS256' });

            const result = await chai
                .request(pn.express)
                .get('/test/isAuthorized')
                .set('Cookie', `authentication_token=${token};refresh_token=abc`);

            result.res.text.should.be.equal('OK');
        });

        it('header', async () => {
            const data = {
                project_name: 'test',
                applications: [
                    {
                        application_code: 'TEST',
                    },
                ],
            };
            const token = jwt.sign(data, PRIVATE_KEY, { algorithm: 'RS256' });

            const result = await chai
                .request(pn.express)
                .get('/test/isAuthorized')
                .set('authentication_token', token)
                .set('refresh_token', 'abc');

            result.res.text.should.be.equal('OK');
        });

        it('header multiple projects', async () => {
            const authenticationToken = jwt.sign(
                {
                    projects: ['test'],
                },
                PRIVATE_KEY,
                { algorithm: 'RS256' },
            );
            const authorizationToken = jwt.sign(
                {
                    projectName: 'test',
                    applicationsPermissions: [
                        {
                            application_code: 'TEST',
                        },
                    ],
                },
                PRIVATE_KEY,
                { algorithm: 'RS256' },
            );

            const result = await chai
                .request(pn.express)
                .get('/test/isAuthorized')
                .set('authentication_token', authenticationToken)
                .set('authorization_token', authorizationToken)
                .set('refresh_token', 'abc');

            result.res.text.should.be.equal('OK');
        });

        it('should return 403 when user tries to get data for project she doesnt have permissions to - v4', async () => {
            const authenticationToken = jwt.sign(
                {
                    projects: ['test'],
                },
                PRIVATE_KEY,
                { algorithm: 'RS256' },
            );

            nock('http://auth.example.com')
                .get('/auth/v4.1/authorization-token?projectName=foo&userId=undefined&refresh_token=abc')
                .reply(400, '{"err": "user is not authorized to this project"}');

            const result = await chai
                .request(pn.express)
                .get('/foo/isAuthorized')
                .set('authentication_token', authenticationToken)
                .set('refresh_token', 'abc');

            result.statusCode.should.equal(403);
            result.body.should.be.deep.equal({
                success: false,
                message: {
                    err: 'user is not authorized to this project',
                },
            });
        });

        it('should return 403 when user tries to get data for project she doesnt have permissions to - v4.1', async () => {
            const authenticationToken = jwt.sign(
                {
                    version: '4.1',
                },
                PRIVATE_KEY,
                { algorithm: 'RS256' },
            );

            nock('http://auth.example.com')
                .get('/auth/v4.1/authorization-token?projectName=foo&userId=undefined&refresh_token=abc')
                .reply(400, '{"err": "user is not authorized to this project"}');

            const result = await chai
                .request(pn.express)
                .get('/foo/isAuthorized')
                .set('authentication_token', authenticationToken)
                .set('refresh_token', 'abc');

            result.statusCode.should.equal(403);
            result.body.should.be.deep.equal({
                success: false,
                message: {
                    err: 'user is not authorized to this project',
                },
            });
        });

        it('header refresh token', async () => {
            const data = {
                project_name: 'test',
                applications: [
                    {
                        application_code: 'TEST',
                    },
                ],
                exp: Math.floor(Date.now() / 1000) + 60,
            };
            const newToken = jwt.sign(data, PRIVATE_KEY, { algorithm: 'RS256' });

            nock('http://auth.example.com')
                .get('/auth/v4/authentication-token?grant_type=refresh_token&refresh_token=abc')
                .reply(200, { access_token: newToken, refresh_token: 'cde' });

            const token = jwt.sign({ exp: Math.floor(Date.now() / 1000) - 1 }, PRIVATE_KEY, { algorithm: 'RS256' });

            const result = await chai
                .request(pn.express)
                .get('/test/isAuthorized')
                .set('authentication_token', token)
                .set('refresh_token', 'abc');

            result.res.text.should.be.equal('OK');
        });

        it('cookie refresh token', async () => {
            const data = {
                project_name: 'test',
                applications: [
                    {
                        application_code: 'TEST',
                    },
                ],
                exp: Math.floor(Date.now() / 1000) + 60,
            };
            const newToken = jwt.sign(data, PRIVATE_KEY, { algorithm: 'RS256' });
            nock('http://auth.example.com')
                .get('/auth/v4/authentication-token?grant_type=refresh_token&refresh_token=abc')
                .reply(200, { access_token: newToken, refresh_token: 'cde' });

            const token = jwt.sign({ exp: Math.floor(Date.now() / 1000) - 1 }, PRIVATE_KEY, { algorithm: 'RS256' });

            const result = await chai
                .request(pn.express)
                .get('/test/isAuthorized')
                .set('Cookie', `authentication_token=${token};refresh_token=abc`);

            result.res.text.should.be.equal('OK');
        });
    });

    describe('API token', () => {
        it('simple', async () => {
            const data = {
                project_name: 'test',
                applications: [
                    {
                        application_code: 'TEST',
                    },
                ],
            };
            const token = jwt.sign(data, pn.config.authentication.authServer.secret);

            nock('http://auth.example.com')
                .post('/auth/v3/token')
                .reply(200, { access_token: token });

            const result = await chai
                .request(pn.express)
                .get('/test/isAuthorized')
                .set('Authorization', `Auth-Token ${Math.random().toString(36)}`);

            result.res.text.should.be.equal('OK');
        });

        it('cached', async () => {
            const data = {
                project_name: 'test',
                applications: [
                    {
                        application_code: 'TEST',
                    },
                ],
            };
            const token = jwt.sign(data, pn.config.authentication.authServer.secret);

            nock('http://auth.example.com')
                .post('/auth/v3/token')
                .reply(200, { access_token: token });

            let result = await chai
                .request(pn.express)
                .get('/test/isAuthorized')
                .set('Authorization', 'Auth-Token abc123123');

            result.res.text.should.be.equal('OK');

            result = await chai
                .request(pn.express)
                .get('/test/isAuthorized')
                .set('Authorization', 'Auth-Token abc123123');

            result.res.text.should.be.equal('OK');
        });

        it('auth server error', async () => {
            nock('http://auth.example.com')
                .post('/auth/v3/token')
                .reply(500);

            const result = await chai
                .request(pn.express)
                .get('/test/isAuthorized')
                .set('Authorization', `Auth-Token ${Math.random().toString(36)}`);

            result.res.text.should.be.equal('{"error_code":null,"error":"","httpCode":500}');
        });

        it('bad prefix', async () => {
            const result = await chai
                .request(pn.express)
                .get('/test/isAuthorized')
                .set('authorization', 'Foo');

            result.body.should.be.deep.equal({ success: false, message: 'not a valid Authorization header' });
            result.statusCode.should.equal(401);
        });
    });

    it('no token', async () => {
        const result = await chai.request(pn.express).get('/test/isAuthorized');

        result.body.should.be.deep.equal({ err: 'missing Authorization:Auth-Token header value' });
        result.statusCode.should.equal(401);
    });

    it('bad token', async () => {
        const result = await chai
            .request(pn.express)
            .get('/test/isAuthorized')
            .set('Cookie', 'access_token=1;refresh_token=1');

        result.body.should.be.deep.equal({
            message: { message: 'jwt malformed', name: 'JsonWebTokenError' },
            success: false,
        });
        result.statusCode.should.equal(401);
    });

    it('server to server auth', async () => {
        nock('https://vault.trax-cloud.com')
            .post('/v1/auth/aws/login')
            .reply(200, { auth: { client_token: 'aaa', lease_duration: 1000 } });

        nock('https://vault.trax-cloud.com')
            .get('/v1/secret/int/projects_secrets/test')
            .reply(200, { data: { key: 'test-secret' } });

        const data = {
            project_name: 'Backend-Service',
            applications: [
                {
                    application_code: 'TEST',
                },
            ],
        };
        const token = jwt.sign(data, pn.config.authentication.authServer.secret);

        const result = await chai
            .request(pn.express)
            .get('/test/isAuthorized')
            .set('auth_token', `${JSON.stringify({ access_token: token, refresh_token: 'abc' })}`)
            .set('project_secret', 'test-secret');

        result.res.text.should.be.equal('OK');
    });

    it('server to server request', async () => {
        nock('https://vault.trax-cloud.com')
            .post('/v1/auth/aws/login')
            .reply(200, { auth: { client_token: 'aaa', lease_duration: 1000 } });

        nock('https://vault.trax-cloud.com')
            .get('/v1/secret/int/projects_secrets/test-project')
            .reply(200, { data: { key: 'test-secret' } });

        const mongo = await pn.db.mongo('projects');
        await mongo.collection('api_keys').deleteMany({});
        await mongo.collection('api_keys').insertOne({
            project_name: 'Backend-Service',
            key: 'test-key',
            user_name: 'test-user@traxretail.com',
            service_name: 'test-service',
        });

        nock('http://test.example.com/')
            .get('/test-project/bla')
            .reply(200, 'OK');

        const result: any = await new Promise((resolve, reject) => {
            pn.utils.serverToServerHttpRequest(
                'test-service',
                'test-project',
                {
                    method: 'GET',
                    url: 'http://test.example.com/test-project/bla',
                    headers: {},
                },
                (err, res) => (err ? reject(err) : resolve(res)),
            );
        });

        result.httpRequest.body.should.be.equal('OK');
    });

    describe('authorizeProject', () => {
        it('no header', async () => {
            const result = await chai.request(pn.express).get('/test/authorizeProject');

            result.status.should.be.eql(200);
            result.body.should.have.property('code').be.eql(pn.AUTH_ERRORS.NO_TOKEN_PROVIDED);
            result.body.should.have.property('message').be.eql('NO_TOKEN_PROVIDED');
        });

        describe('API Token', () => {
            it('api token invalid token structure', async () => {
                const result = await chai
                    .request(pn.express)
                    .get('/test/authorizeProject')
                    .set('Authorization', 'invalid');

                result.status.should.be.eql(200);
                result.body.should.have.property('code').be.eql(pn.AUTH_ERRORS.INVALID_API_TOKEN);
                result.body.should.have.property('message').be.eql('INVALID_API_TOKEN');
            });

            it('api token invalid token data', async () => {
                nock('http://auth.example.com')
                    .post('/auth/v3/token')
                    .reply(400, {
                        err: { error: 'invalid_grant', error_description: 'The refresh token provided is not valid' },
                    });

                const result = await chai
                    .request(pn.express)
                    .get('/test/authorizeProject')
                    .set('Authorization', 'Auth-Token 123');

                result.status.should.be.eql(200);
                result.body.should.have.property('code').be.eql(pn.AUTH_ERRORS.INVALID_TOKEN);
                result.body.should.have.property('message').be.eql('INVALID_TOKEN');
            });

            it('api token invalid secret', async () => {
                const token = jwt.sign({ project_name: 'test' }, pn.config.authentication.authServer.secret + '!');

                nock('http://auth.example.com')
                    .post('/auth/v3/token')
                    .reply(200, { access_token: token });

                const result = await chai
                    .request(pn.express)
                    .get('/test/authorizeProject')
                    .set('Authorization', 'Auth-Token 123');

                result.status.should.be.eql(200);
                result.body.should.have.property('code').be.eql(pn.AUTH_ERRORS.INVALID_JWT_SIGNITURE);
                result.body.should.have.property('message').be.eql('INVALID_JWT_SIGNITURE');
            });

            it('api token jwt expired without refresh token', async () => {
                const token = jwt.sign({ project_name: 'test' }, pn.config.authentication.authServer.secret, { expiresIn: -1 });

                nock('http://auth.example.com')
                    .post('/auth/v3/token')
                    .reply(200, { access_token: token });

                const result = await chai
                    .request(pn.express)
                    .get('/test/authorizeProject')
                    .set(
                        'Authorization',
                        'Auth-Token 5eVOcY7quBfgeA1tvOGOgcpMKw3B7v4fS4XSOIl7GuY2LC1IvoqEaEZhtCagZIDdWLHW2k76JeoP5wpCgEaReCv2Cv8qId4E0CilGUhcpOPQTBm9eyUookl0gupowA8e4CNPLaMQ8H55ACra2MykdU0P',
                    );

                result.status.should.be.eql(200);
                result.body.should.have.property('code').be.eql(pn.AUTH_ERRORS.NO_REFRESH_TOKEN_PROVIDED);
                result.body.should.have.property('message').be.eql('NO_REFRESH_TOKEN_PROVIDED');
            });

            it('api token unauthorized project', async () => {
                const token = jwt.sign({ project_name: 'test' }, pn.config.authentication.authServer.secret);

                nock('http://auth.example.com')
                    .post('/auth/v3/token')
                    .reply(200, { access_token: token });

                const result = await chai
                    .request(pn.express)
                    .get('/test1/authorizeProject')
                    .set('Authorization', 'Auth-Token 123');

                result.status.should.be.eql(200);
                result.body.should.have.property('code').be.eql(pn.AUTH_ERRORS.UNAUTHORIZED_PROJECT);
                result.body.should.have.property('message').be.eql('UNAUTHORIZED_PROJECT');
            });

            it('api token in Authorization header', async () => {
                const token = jwt.sign(
                    { project_name: 'test', user_name: 'test_user' },
                    pn.config.authentication.authServer.secret,
                );

                nock('http://auth.example.com')
                    .post('/auth/v3/token')
                    .reply(200, { access_token: token });

                const result = await chai
                    .request(pn.express)
                    .get('/test/authorizeProject')
                    .set('Authorization', 'Auth-Token 123');

                result.status.should.be.eql(200);
                result.body.should.have.property('user');
                result.body.user.should.have.property('project_name').to.be.eql('test');
                result.body.user.should.have.property('user_name').to.be.eql('test_user');
            });

            it('api token in authorization header', async () => {
                const token = jwt.sign(
                    { project_name: 'test', user_name: 'test_user' },
                    pn.config.authentication.authServer.secret,
                );

                nock('http://auth.example.com')
                    .post('/auth/v3/token')
                    .reply(200, { access_token: token });

                const result = await chai
                    .request(pn.express)
                    .get('/test/authorizeProject')
                    .set('authorization', 'Auth-Token 123');

                result.status.should.be.eql(200);
                result.body.should.have.property('user');
                result.body.user.should.have.property('project_name').to.be.eql('test');
                result.body.user.should.have.property('user_name').to.be.eql('test_user');
            });

            it('api token in authorization header with spaces', async () => {
                const token = jwt.sign(
                    { project_name: 'test', user_name: 'test_user' },
                    pn.config.authentication.authServer.secret,
                );

                nock('http://auth.example.com')
                    .post('/auth/v3/token')
                    .reply(200, { access_token: token });

                const result = await chai
                    .request(pn.express)
                    .get('/test/authorizeProject')
                    .set('authorization', ' Auth-Token 123');

                result.status.should.be.eql(200);
                result.body.should.have.property('user');
                result.body.user.should.have.property('project_name').to.be.eql('test');
                result.body.user.should.have.property('user_name').to.be.eql('test_user');
            });

            it('backend-service should fail if no project_secret provided', async () => {
                const token = jwt.sign(
                    { project_name: 'Backend-Service', user_name: 'test_user' },
                    pn.config.authentication.authServer.secret,
                );

                nock('http://auth.example.com')
                    .post('/auth/v3/token')
                    .reply(200, { access_token: token });

                const result = await chai
                    .request(pn.express)
                    .get('/test/authorizeProject')
                    .set('authorization', ' Auth-Token 123');

                result.status.should.be.eql(200);
                result.body.should.have.property('code').be.eql(pn.AUTH_ERRORS.NO_SECRET_PROVIDED);
                result.body.should.have.property('message').be.eql('NO_SECRET_PROVIDED');
            });

            it('backend-service should fail on invalid project_secret provided', async () => {
                const token = jwt.sign(
                    { project_name: 'Backend-Service', user_name: 'test_user' },
                    pn.config.authentication.authServer.secret,
                );

                nock('http://auth.example.com')
                    .post('/auth/v3/token')
                    .reply(200, { access_token: token });

                nock('https://vault.trax-cloud.com')
                    .post('/v1/auth/aws/login')
                    .reply(200, { auth: { client_token: 'aaa', lease_duration: 1000 } })
                    .get('/v1/secret/int/projects_secrets/test')
                    .reply(200, { data: { key: 'test-secret' } });

                const result = await chai
                    .request(pn.express)
                    .get('/test/authorizeProject')
                    .set('authorization', ' Auth-Token 123')
                    .set('project_secret', '#secret#');

                result.status.should.be.eql(200);
                result.body.should.have.property('code').be.eql(pn.AUTH_ERRORS.INVALID_API_SECRET);
                result.body.should.have.property('message').be.eql('INVALID_API_SECRET');
            });

            it('backend-service should get user with provided', async () => {
                const token = jwt.sign(
                    { project_name: 'Backend-Service', user_name: 'test_user' },
                    pn.config.authentication.authServer.secret,
                );

                nock('http://auth.example.com')
                    .post('/auth/v3/token')
                    .reply(200, { access_token: token });

                nock('https://vault.trax-cloud.com')
                    .post('/v1/auth/aws/login')
                    .reply(200, { auth: { client_token: 'aaa', lease_duration: 1000 } })
                    .get('/v1/secret/int/projects_secrets/test')
                    .reply(200, { data: { key: '#secret#' } });

                const result = await chai
                    .request(pn.express)
                    .get('/test/authorizeProject')
                    .set('authorization', ' Auth-Token 123')
                    .set('project_secret', '#secret#');

                result.status.should.be.eql(200);
                result.body.should.have.property('user');
                result.body.user.should.have.property('project_name').be.eql('Backend-Service');
                result.body.user.should.have.property('user_name').be.eql('test_user');
            });
        });

        describe('V4', () => {
            it('v4 invalid token', async () => {
                const result = await chai
                    .request(pn.express)
                    .get('/test/authorizeProject')
                    .set('authentication_token', 'invalid');

                result.status.should.be.eql(200);
                result.body.should.have.property('code').be.eql(pn.AUTH_ERRORS.INVALID_TOKEN);
                result.body.should.have.property('message').be.eql('INVALID_TOKEN');
            });

            it('v4 no projects assigned to user', async () => {
                const token = jwt.sign({ project_name: 'test' }, PRIVATE_KEY, { algorithm: 'RS256' });
                const result = await chai
                    .request(pn.express)
                    .get('/test/authorizeProject')
                    .set('authentication_token', token);

                result.status.should.be.eql(200);
                result.body.should.have.property('code').be.eql(pn.AUTH_ERRORS.UNAUTHORIZED_NO_PROJECTS);
                result.body.should.have.property('message').be.eql('UNAUTHORIZED_NO_PROJECTS');
            });

            it('v4.1 no projects assigned to user', async () => {
                const token = jwt.sign({ project_name: 'test', version: '4.1' }, PRIVATE_KEY, { algorithm: 'RS256' });
                const result = await chai
                    .request(pn.express)
                    .get('/test/authorizeProject')
                    .set('authentication_token', token);

                result.status.should.be.eql(200);
                result.body.should.have.property('user');
                result.body.user.should.have.property('project_name').be.eql('test');
            });

            it('v4 should be authenticated user for a valid token with projects', async () => {
                const token = jwt.sign({ project_name: 'test', projects: [] }, PRIVATE_KEY, { algorithm: 'RS256' });
                const result = await chai
                    .request(pn.express)
                    .get('/test/authorizeProject')
                    .set('authentication_token', token);

                result.status.should.be.eql(200);
                result.body.should.have.property('user');
                result.body.user.should.have.property('project_name').be.eql('test');
                result.body.user.should.have.property('projects').be.eql([]);
            });

            it('v4 should be authenticated user for a valid token with cookies', async () => {
                const token = jwt.sign({ project_name: 'test', projects: [] }, PRIVATE_KEY, { algorithm: 'RS256' });
                const result = await chai
                    .request(pn.express)
                    .get('/test/authorizeProject')
                    .set('Cookie', `authentication_token=${token}`);

                result.status.should.be.eql(200);
                result.body.should.have.property('user');
                result.body.user.should.have.property('project_name').be.eql('test');
                result.body.user.should.have.property('projects').be.eql([]);
            });

            it('v4 should get new token on jwt expired with refresh token', async () => {
                const token = jwt.sign({ projects: ['test'] }, PRIVATE_KEY, { expiresIn: -1, algorithm: 'RS256' });
                const newToken = jwt.sign({ projects: ['test'] }, PRIVATE_KEY, { algorithm: 'RS256' });
                const newRefreshToken = jwt.sign({ project_name: 'test' }, PRIVATE_KEY, { algorithm: 'RS256' });

                nock('http://auth.example.com')
                    .post('/auth/v3/updateUserPush')
                    .reply(200)
                    .get('/auth/v4/authentication-token')
                    .query({
                        grant_type: 'refresh_token',
                        refresh_token: '123',
                    })
                    .reply(200, { access_token: newToken, refresh_token: newRefreshToken });

                const result = await chai
                    .request(pn.express)
                    .get('/test/authorizeProject')
                    .set('authentication_token', token)
                    .set('refresh_token', '123');

                result.status.should.be.eql(200);
                result.body.should.have.property('user');
                result.body.should.have.property('fromCookie').eql(false);
                result.body.should.have.property('tokens');
                result.body.tokens.should.have.property('access_token').eql(newToken);
                result.body.tokens.should.have.property('refresh_token').eql(newRefreshToken);
            });

            it('v4 should get new token on jwt expired cookie with refresh token', async () => {
                const token = jwt.sign({ projects: ['test'] }, PRIVATE_KEY, { expiresIn: -1, algorithm: 'RS256' });
                const newToken = jwt.sign({ projects: ['test'] }, PRIVATE_KEY, { algorithm: 'RS256' });
                const newRefreshToken = jwt.sign({ project_name: 'test' }, PRIVATE_KEY, { algorithm: 'RS256' });

                nock('http://auth.example.com')
                    .post('/auth/v3/updateUserPush')
                    .reply(200)
                    .get('/auth/v4/authentication-token')
                    .query({
                        grant_type: 'refresh_token',
                        refresh_token: '123',
                    })
                    .reply(200, { access_token: newToken, refresh_token: newRefreshToken });

                const result = await chai
                    .request(pn.express)
                    .get('/test/authorizeProject')
                    .set('Cookie', `authentication_token=${token};refresh_token=123`);

                result.status.should.be.eql(200);
                result.body.should.have.property('user');
                result.body.should.have.property('fromCookie').eql(true);
                result.body.should.have.property('tokens');
                result.body.tokens.should.have.property('access_token').eql(newToken);
                result.body.tokens.should.have.property('refresh_token').eql(newRefreshToken);
            });

            it('v4 should fire refresh event to auth server', async () => {
                const user = { projects: ['test'] };
                const now = Date.now();
                const login_session = '1111';

                const token = jwt.sign(user, PRIVATE_KEY, { expiresIn: -1, algorithm: 'RS256' });
                const newToken = jwt.sign({ ...user, exp: now, login_session }, PRIVATE_KEY, { algorithm: 'RS256' });
                const newRefreshToken = jwt.sign(user, PRIVATE_KEY, { algorithm: 'RS256' });

                nock('http://auth.example.com')
                    .post('/auth/v3/updateUserPush', {
                        old_token_login_session: '',
                        new_token_login_session: login_session,
                        exp_date: moment.unix(now).toISOString(),
                    })
                    .reply(200)
                    .get('/auth/v4/authentication-token')
                    .query({
                        grant_type: 'refresh_token',
                        refresh_token: '123',
                    })
                    .reply(200, { access_token: newToken, refresh_token: newRefreshToken });

                const response = await chai
                    .request(pn.express)
                    .get('/test/authorizeProject')
                    .set('authentication_token', token)
                    .set('refresh_token', '123');

                response.status.should.be.eql(200);
            });
        });

        describe('V3', () => {
            it('v3 invalid token', async () => {
                const result = await chai
                    .request(pn.express)
                    .get('/test/authorizeProject')
                    .set('auth_token', 'invalid');

                result.status.should.be.eql(200);
                result.body.should.have.property('code').be.eql(pn.AUTH_ERRORS.INVALID_TOKEN);
                result.body.should.have.property('message').be.eql('INVALID_TOKEN');
            });

            it('v3 unauthenticated - user cannot access other project', async () => {
                const token = jwt.sign({ project_name: 'test' }, pn.config.authentication.authServer.secret);
                const result = await chai
                    .request(pn.express)
                    .get('/test1/authorizeProject')
                    .set('auth_token', token);

                result.status.should.be.eql(200);
                result.body.should.have.property('code').be.eql(pn.AUTH_ERRORS.UNAUTHORIZED_PROJECT);
                result.body.should.have.property('message').be.eql('UNAUTHORIZED_PROJECT');
            });

            it('v3 should be authenticated user for a valid token with project', async () => {
                const token = jwt.sign({ project_name: 'test' }, pn.config.authentication.authServer.secret);
                const result = await chai
                    .request(pn.express)
                    .get('/test/authorizeProject')
                    .set('auth_token', token);

                result.status.should.be.eql(200);
                result.body.should.have.property('user');
                result.body.user.should.have.property('project_name').be.eql('test');
            });

            it('v3 should be authenticated user for a valid token with cookies', async () => {
                const token = jwt.sign({ project_name: 'test' }, pn.config.authentication.authServer.secret);
                const result = await chai
                    .request(pn.express)
                    .get('/test/authorizeProject')
                    .set('Cookie', `auth_token=${token}`);

                result.status.should.be.eql(200);
                result.body.should.have.property('user');
                result.body.user.should.have.property('project_name').be.eql('test');
            });

            it('v3 should get new token on jwt expired with refresh token', async () => {
                const user = { project_name: 'test' };
                const token = jwt.sign(user, pn.config.authentication.authServer.secret, { expiresIn: -1 });
                const newToken = jwt.sign(user, pn.config.authentication.authServer.secret);
                const newRefreshToken = jwt.sign(user, pn.config.authentication.authServer.secret);

                nock('http://auth.example.com')
                    .post('/auth/v3/updateUserPush')
                    .reply(200)
                    .post('/auth/v3/token')
                    .reply(200, { access_token: newToken, refresh_token: newRefreshToken });

                const result = await chai
                    .request(pn.express)
                    .get('/test/authorizeProject')
                    .set('auth_token', token)
                    .set('refresh_token', '123');

                result.status.should.be.eql(200);
                result.body.should.have.property('user');
                result.body.should.have.property('fromCookie').eql(false);
                result.body.should.have.property('tokens');
                result.body.tokens.should.have.property('access_token').eql(newToken);
                result.body.tokens.should.have.property('refresh_token').eql(newRefreshToken);
            });

            it('v3 should get new token on jwt expired cookie with refresh token', async () => {
                const user = { project_name: 'test' };
                const token = jwt.sign(user, pn.config.authentication.authServer.secret, { expiresIn: -1 });
                const newToken = jwt.sign(user, pn.config.authentication.authServer.secret);
                const newRefreshToken = jwt.sign(user, pn.config.authentication.authServer.secret);

                nock('http://auth.example.com')
                    .post('/auth/v3/updateUserPush')
                    .reply(200)
                    .post('/auth/v3/token')
                    .reply(200, { access_token: newToken, refresh_token: newRefreshToken });

                const result = await chai
                    .request(pn.express)
                    .get('/test/authorizeProject')
                    .set('Cookie', `auth_token=${token};refresh_token=123`);

                result.status.should.be.eql(200);
                result.body.should.have.property('user');
                result.body.should.have.property('fromCookie').eql(true);
                result.body.should.have.property('tokens');
                result.body.tokens.should.have.property('access_token').eql(newToken);
                result.body.tokens.should.have.property('refresh_token').eql(newRefreshToken);
            });
        });
    });

    describe('authorize', () => {
        describe('V3', () => {
            it('v3 should get 401 for unauthorized user without token in header', async () => {
                const response = await chai.request(pn.express).get('/test/authorize');

                response.status.should.be.eql(401);
                response.body.should.have.property('error').eql('NO_TOKEN_PROVIDED');
                response.body.should.have.property('code').eql(10);
                response.headers.should.not.have.property('auth_token');
            });

            it('v3 should set token in header', async () => {
                const token = jwt.sign({ project_name: 'test' }, pn.config.authentication.authServer.secret);

                const response = await chai
                    .request(pn.express)
                    .get('/test/authorize')
                    .set('auth_token', token);

                response.status.should.be.eql(200);
                response.body.should.have.property('project_name').eql('test');
                response.headers.should.have.property('auth_token').eql(token);
            });

            it('v3 should get 403 with user without permissions', async () => {
                const token = jwt.sign({ project_name: 'test' }, pn.config.authentication.authServer.secret);

                const response = await chai
                    .request(pn.express)
                    .get('/test/authorize/permissions')
                    .set('auth_token', token);

                response.status.should.be.eql(403);
            });

            it('v3 should get 200 for user with string permission', async () => {
                const user = {
                    project_name: 'test',
                    applications: [
                        {
                            application_code: 'TEST',
                        },
                    ],
                };
                const token = jwt.sign(user, pn.config.authentication.authServer.secret);

                const response = await chai
                    .request(pn.express)
                    .get('/test/authorize/permissions')
                    .set('auth_token', token);

                response.status.should.be.eql(200);
            });

            it('v3 should get 403 for user with application permission and without entity permission', async () => {
                const user = {
                    project_name: 'test',
                    applications: [
                        {
                            application_code: 'TEST2',
                            entities: [
                                {
                                    entity_name: 'NO_ACCESS',
                                },
                            ],
                        },
                    ],
                };
                const token = jwt.sign(user, pn.config.authentication.authServer.secret);

                const response = await chai
                    .request(pn.express)
                    .get('/test/authorize/permissions')
                    .set('auth_token', token);

                response.status.should.be.eql(403);
            });

            it('v3 should get 403 for user with application permission and entity permission without permissions', async () => {
                const user = {
                    project_name: 'test',
                    applications: [
                        {
                            application_code: 'TEST2',
                            entities: [
                                {
                                    entity_name: 'ENTITY',
                                    permissions: [],
                                },
                            ],
                        },
                    ],
                };
                const token = jwt.sign(user, pn.config.authentication.authServer.secret);

                const response = await chai
                    .request(pn.express)
                    .get('/test/authorize/permissions')
                    .set('auth_token', token);

                response.status.should.be.eql(403);
            });

            it('v3 should get 200 for user with application permission and entity permission', async () => {
                const user = {
                    project_name: 'test',
                    applications: [
                        {
                            application_code: 'TEST2',
                            entities: [
                                {
                                    entity_name: 'Entity',
                                    permissions: ['View'],
                                },
                            ],
                        },
                    ],
                };
                const token = jwt.sign(user, pn.config.authentication.authServer.secret);

                const response = await chai
                    .request(pn.express)
                    .get('/test/authorize/permissions')
                    .set('auth_token', token);

                response.status.should.be.eql(200);
            });

            it('v3 should set token in header for expired token', async () => {
                const user = { project_name: 'test' };
                const token = jwt.sign(user, pn.config.authentication.authServer.secret, { expiresIn: -1 });
                const newToken = jwt.sign(user, pn.config.authentication.authServer.secret);
                const newRefreshToken = jwt.sign(user, pn.config.authentication.authServer.secret);

                nock('http://auth.example.com')
                    .post('/auth/v3/updateUserPush')
                    .reply(200)
                    .post('/auth/v3/token')
                    .reply(200, { access_token: newToken, refresh_token: newRefreshToken });

                const response = await chai
                    .request(pn.express)
                    .get('/test/authorize')
                    .set('auth_token', token)
                    .set('refresh_token', '123');

                response.status.should.be.eql(200);
                response.body.should.have.property('project_name').eql('test');
                response.headers.should.have.property('auth_token').eql(newToken);
            });

            it('v3 should return the authenticated user without project validation', async () => {
                const user = {
                    user_id: 1,
                    project_name: 'test',
                };
                const token = jwt.sign(user, pn.config.authentication.authServer.secret);

                const response = await chai
                    .request(pn.express)
                    .get('/authorize/my-user')
                    .set('auth_token', token);

                response.status.should.be.eql(200);
                response.body.should.have.property('user_id').eql(1);
                response.body.should.have.property('iat');
                response.body.should.have.property('project_name').eql('test');
            });
        });

        describe('V4', () => {
            it('v4 should get 401 for unauthorized user without token in header', async () => {
                const response = await chai.request(pn.express).get('/test/authorize');

                response.status.should.be.eql(401);
                response.body.should.have.property('error').eql('NO_TOKEN_PROVIDED');
                response.body.should.have.property('code').eql(10);
                response.headers.should.not.have.property('authentication_token');
            });

            it('v4 should set token in header', async () => {
                const token = jwt.sign({ projects: ['test'] }, PRIVATE_KEY, { algorithm: 'RS256' });

                const response = await chai
                    .request(pn.express)
                    .get('/test/authorize')
                    .set('authentication_token', token);

                response.status.should.be.eql(200);
                response.body.should.have.property('projects');
                response.body.projects[0].should.be.eql('test');
                response.headers.should.have.property('authentication_token').eql(token);
            });

            it('v4 should get 403 for auth server bad response', async () => {
                const user = {
                    user_id: 1,
                    projects: ['test'],
                };
                const token = jwt.sign(user, PRIVATE_KEY, { algorithm: 'RS256' });

                nock('http://auth.example.com')
                    .get('/auth/v4.1/authorization-token')
                    .query(true)
                    .reply(400);

                const response = await chai
                    .request(pn.express)
                    .get('/test/authorize/permissions')
                    .set('authentication_token', token)
                    .set('refresh_token', '123');

                response.status.should.be.eql(403);
            });

            it('v4 should get 200 for user with valid string permission', async () => {
                const user = {
                    user_id: 1,
                    projects: ['test'],
                };
                const authUser = {
                    projectName: 'test',
                    applicationsPermissions: [
                        {
                            application_code: 'TEST',
                        },
                    ],
                };
                const token = jwt.sign(user, PRIVATE_KEY, { algorithm: 'RS256' });
                const authToken = jwt.sign(authUser, PRIVATE_KEY, { algorithm: 'RS256' });

                nock('http://auth.example.com')
                    .get('/auth/v4.1/authorization-token')
                    .query(true)
                    .reply(200, null, { authorization_token: authToken });

                const response = await chai
                    .request(pn.express)
                    .get('/test/authorize/permissions')
                    .set('authentication_token', token)
                    .set('refresh_token', '123');

                response.status.should.be.eql(200);
            });

            it('v4 should get 403 for user with no app permissions', async () => {
                const user = {
                    user_id: 1,
                    projects: ['test'],
                };
                const authUser = {
                    projectName: 'test',
                    applicationsPermissions: [
                        {
                            application_code: 'TEST3',
                        },
                    ],
                };
                const token = jwt.sign(user, PRIVATE_KEY, { algorithm: 'RS256' });
                const authToken = jwt.sign(authUser, PRIVATE_KEY, { algorithm: 'RS256' });

                nock('http://auth.example.com')
                    .get('/auth/v4.1/authorization-token')
                    .query(true)
                    .reply(200, null, { authorization_token: authToken });

                const response = await chai
                    .request(pn.express)
                    .get('/test/authorize/permissions')
                    .set('authentication_token', token)
                    .set('refresh_token', '123');

                response.status.should.be.eql(403);
            });

            it('v4 should get 403 for user with no entity permissions', async () => {
                const user = {
                    user_id: 1,
                    projects: ['test'],
                };
                const authUser = {
                    projectName: 'test',
                    applicationsPermissions: [
                        {
                            application_code: 'TEST2',
                            entities: [
                                {
                                    name: 'Entity2',
                                },
                            ],
                        },
                    ],
                };
                const token = jwt.sign(user, PRIVATE_KEY, { algorithm: 'RS256' });
                const authToken = jwt.sign(authUser, PRIVATE_KEY, { algorithm: 'RS256' });

                nock('http://auth.example.com')
                    .get('/auth/v4.1/authorization-token')
                    .query(true)
                    .reply(200, null, { authorization_token: authToken });

                const response = await chai
                    .request(pn.express)
                    .get('/test/authorize/permissions')
                    .set('authentication_token', token)
                    .set('refresh_token', '123');

                response.status.should.be.eql(403);
            });

            it('v4 should get 403 for user with no entity specific permission', async () => {
                const user = {
                    user_id: 1,
                    projects: ['test'],
                };
                const authUser = {
                    projectName: 'test',
                    applicationsPermissions: [
                        {
                            application_code: 'TEST2',
                            entities: [
                                {
                                    name: 'Entity',
                                    permissions: ['NO_VIEW'],
                                },
                            ],
                        },
                    ],
                };
                const token = jwt.sign(user, PRIVATE_KEY, { algorithm: 'RS256' });
                const authToken = jwt.sign(authUser, PRIVATE_KEY, { algorithm: 'RS256' });

                nock('http://auth.example.com')
                    .get('/auth/v4.1/authorization-token')
                    .query(true)
                    .reply(200, null, { authorization_token: authToken });

                const response = await chai
                    .request(pn.express)
                    .get('/test/authorize/permissions')
                    .set('authentication_token', token)
                    .set('refresh_token', '123');

                response.status.should.be.eql(403);
            });

            it('v4 should get 200 for user with entity specific permission', async () => {
                const user = {
                    user_id: 1,
                    projects: ['test'],
                };
                const authUser = {
                    projectName: 'test',
                    applicationsPermissions: [
                        {
                            application_code: 'TEST2',
                            entities: [
                                {
                                    name: 'Entity',
                                    permissions: ['View'],
                                },
                            ],
                        },
                    ],
                };
                const token = jwt.sign(user, PRIVATE_KEY, { algorithm: 'RS256' });
                const authToken = jwt.sign(authUser, PRIVATE_KEY, { algorithm: 'RS256' });

                nock('http://auth.example.com')
                    .get('/auth/v4.1/authorization-token')
                    .query(true)
                    .reply(200, null, { authorization_token: authToken });

                const response = await chai
                    .request(pn.express)
                    .get('/test/authorize/permissions')
                    .set('authentication_token', token)
                    .set('refresh_token', '123');

                response.status.should.be.eql(200);
                response.headers.should.have.property('authorization_token').eql(authToken);
            });

            it('v4 should set token in header for expired token', async () => {
                const token = jwt.sign({ projects: ['test'] }, PRIVATE_KEY, { expiresIn: -1, algorithm: 'RS256' });
                const newToken = jwt.sign({ projects: ['test'] }, PRIVATE_KEY, { algorithm: 'RS256' });
                const newRefreshToken = jwt.sign({ project_name: 'test' }, PRIVATE_KEY, { algorithm: 'RS256' });

                nock('http://auth.example.com')
                    .post('/auth/v3/updateUserPush')
                    .reply(200)
                    .get('/auth/v4/authentication-token')
                    .query({
                        grant_type: 'refresh_token',
                        refresh_token: '123',
                    })
                    .reply(200, { access_token: newToken, refresh_token: newRefreshToken });

                const response = await chai
                    .request(pn.express)
                    .get('/test/authorize')
                    .set('authentication_token', token)
                    .set('refresh_token', '123');

                response.status.should.be.eql(200);
                response.body.should.have.property('projects');
                response.body.projects[0].should.be.eql('test');
                response.headers.should.have.property('authentication_token').eql(newToken);
            });

            it('v4 should return the authenticated user without project validation', async () => {
                const user = {
                    user_id: 1,
                    projects: ['test'],
                };
                const authUser = {
                    projectName: 'test',
                    applicationsPermissions: [
                        {
                            application_code: 'TEST2',
                            entities: [
                                {
                                    name: 'Entity',
                                    permissions: ['View'],
                                },
                            ],
                        },
                    ],
                };
                const token = jwt.sign(user, PRIVATE_KEY, { algorithm: 'RS256' });
                const authToken = jwt.sign(authUser, PRIVATE_KEY, { algorithm: 'RS256' });

                nock('http://auth.example.com')
                    .get('/auth/v4.1/authorization-token')
                    .query(true)
                    .reply(200, null, { authorization_token: authToken });

                const response = await chai
                    .request(pn.express)
                    .get('/authorize/my-user')
                    .set('authentication_token', token)
                    .set('refresh_token', '123');

                response.status.should.be.eql(200);
                response.body.should.have.property('user_id').eql(1);
                response.body.should.have.property('iat');
                response.body.should.have
                    .property('projects')
                    .and.be.instanceOf(Array)
                    .with.lengthOf(1);
                response.body.projects[0].should.be.eql('test');
            });
        });
    });
});
