
import {promisify} from 'util';
import {Service} from "pn-ts/decorators";
const pn = require('pn-ts')();

@Service()
export class APIService {

    get(url,projectName) {
        return this.request('GET', projectName, pn.config.Url + url);
    }

    post(url, projectName) {
        return this.request('POST', projectName, pn.config.Url + url);
    }

    put(url, projectName) {
        return this.request('PUT', projectName, pn.config.Url + url);
    }

    async request(method: string, projectName, url) {
        const reqSettings = {method: 'GET', url, headers: {'Content-Type': 'application/json',}, json: true};
        try {
            const res = await promisify(pn.utils.serverToServerHttpRequest)(pn.config.serviceName, projectName, reqSettings);
            return JSON.parse(res.httpRequest.body);
        } catch (err) {
            pn.logger.error(
                {
                    err,
                },
                'Error getting data from ' + pn.config.Url,
            );

            throw err;
        }
    }
}
