export {};
const { createTestClient } = require('apollo-server-testing');
const chai = require('chai');
const chaiGraphQL = require('chai-graphql');
const chaiHttp = require('chai-http');

chai.use(chaiHttp);
chai.use(chaiGraphQL);

chai.should();

describe('components/graphql', async () => {
    let _query;
    let _pn;

    before(() => {
        _pn = require('../../src')('graphql-example');
        _pn.init(__dirname, 'TEST');

        const { query } = createTestClient(_pn.apolloServer);
        _query = query;
    });

    it('should retuen "world" on "hello" query', async () => {
        const response = await _query({
            query: '{hello}',
        });

        chai.assert.graphQL(response, { hello: 'world' });
    });

    it('should load datasources and return users from external API', async () => {
        const response = await _query({
            query: '{users{id username email}}',
        });

        chai.assert.graphQLSubset(response, { users: [{ id: 1, username: 'Bret', email: 'Sincere@april.biz' }] });
        chai.assert.lengthOf(response.data.users, 10);
    });

    it('should load custom directives and apply them. upper case directive to work', async () => {
        const response = await _query({
            query: '{toTitle(word: "uppercase me")}',
        });

        chai.assert.graphQL(response, { toTitle: 'UPPERCASE ME!' });
    });

    it('isalive', async () => {
        const result = await chai.request(_pn.express).get(`${_pn.config.express.apiPrefix}/isalive`);
        result.res.text.should.be.equal('OK');
    });
});
