export {};
const pn = require('../../src')('sql-example'); // require('progressive-node')('<app name>');

pn.init(__dirname, 'TEST');

const QueryTypes = pn.Sequelize.QueryTypes;
const chai = require('chai');
chai.should();

describe('components/sql', () => {
    it('model', async () => {
        const mongo = await pn.db.mongo('projects');
        await mongo.collection('projects_project').deleteMany({})
        await mongo.collection('projects_project').insertOne({
            project_name: 'test',
        });

        const sql = await pn.db.sql('test')
        const { Test } = sql.models;

        Test.should.have.property('tableName').equal('test');
    });

    it('query postgres', async () => {
        const mongo = await pn.db.mongo('projects');
        await mongo.collection('projects_project').deleteMany({});
        await mongo.collection('projects_project').insertOne({
            project_name: 'test',
            postgresql: {
                intelligence: {
                    host: '10.3.3.102',
                    database: 'example',
                },
            },
        });

        const sql = await pn.db.sql('test', 'intelligence');

        (await sql.query('SELECT 12+7 AS a;', { type: QueryTypes.SELECT })).should.be.deep.equal([{ a: 19 }]);
    });

    it('query', async () => {
        const mongo = await pn.db.mongo('projects');
        await mongo.collection('projects_project').deleteMany({});
        await mongo.collection('projects_project').insertOne({
            project_name: 'test',
            rds_name: '10.3.3.101',
        });

        const sql = await pn.db.sql('test');

        (await sql.query('SELECT 12+7 AS a;', { type: QueryTypes.SELECT })).should.be.deep.equal([{ a: 19 }]);
    });

    it('query with other mongo', async () => {
        const mongo = await pn.db.mongo('other_projects');
        await mongo.collection('projects_project').deleteMany({});
        await mongo.collection('projects_project').insertOne({
            project_name: 'test',
            rds_name: '10.3.3.101',
        });

        const sql = await pn.db.sql('test', 'projects', 'test_projects');

        (await sql.query('SELECT 12+7 AS a;', { type: QueryTypes.SELECT })).should.be.deep.equal([{ a: 19 }]);
    });

    it('query with Single Connection', async () => {
        const mongo = await pn.db.mongo('projects');
        await mongo.collection('projects_project').deleteMany({});
        await mongo.collection('projects_project').insertOne({
            project_name: 'test',
            rds_name: '10.3.3.101',
        });
        const singleConnection = await pn.db.sqlQuerySingleConnection('test', 'SELECT 12+7 AS res;', { type: QueryTypes.SELECT });
        singleConnection.should.be.deep.equal([{ res: 19 }]);
    });

    it('should throw for unknown project', async () => {
        try {
            await pn.db.sql('unknown');
        } catch (err) {
            err.message.should.includes('project unknown not found');
        }
    });

    it('should throw for project without postgress', async () => {
        const mongo = await pn.db.mongo('projects');
        await mongo.collection('projects_project').deleteMany({});
        await mongo.collection('projects_project').insertOne({
            project_name: 'test',
            postgresql: {},
        });

        try {
            await pn.db.sql('test', 'intelligence');
        } catch (err) {
            err.message.should.includes('postgres for test not found')
        }
    });



    it('direct MySQL query', async () => {
        const mongo = await pn.db.mongo('projects');
        await mongo.collection('projects_project').deleteMany({});

        const sql = await pn.db.sql('abc');

        (await sql.query('SELECT 12+7 AS a;', { type: QueryTypes.SELECT })).should.be.deep.equal([{ a: 19 }]);
    });

    it('should create single sql connection pool when initializing sql in parallel', async () => {
        const mongo = await pn.db.mongo('projects');
        await mongo.collection('projects_project').deleteMany({});
        // adding test project
        await mongo.collection('projects_project').insertOne({
            project_name: 'test',
            rds_name: '10.3.3.101',
        });
        const res = await Promise.all([pn.db.sql('test'), pn.db.sql('test')]);
        res[0].should.be.equal(res[1]);
    });

    it('should throw an error when giving a wrong rds name (unsuccessful connection)', async () => {
        const mongo = await pn.db.mongo('projects');
        await mongo.collection('projects_project').deleteMany({});
        // adding test project
        await mongo.collection('projects_project').insertOne({
            project_name: 'test',
            rds_name: '10.3.3.102', // SHOULD BE 10.3.3.101
        });
        try {
            await pn.db.sql('test');
        } catch (err) {
            err.message.should.to.include('Progressive Node [sql]: test health check failed while CREATING a pool');
        }
    });

    it('sqlQuerySingleConnection - should throw an error when giving a wrong rds name on (unsuccessful connection)', async () => {
        const mongo = await pn.db.mongo('projects');
        await mongo.collection('projects_project').deleteMany({});
        // adding test project
        await mongo.collection('projects_project').insertOne({
            project_name: 'test',
            rds_name: '10.3.3.102', // SHOULD BE 10.3.3.101
        });
        try {
            await pn.db.sqlQuerySingleConnection('test', 'SELECT 1+1 AS result', {});
        } catch (err) {
            err.message.should.to.include('Progressive Node [sql]: test health check failed while CREATING a pool');
        }
    });
});
